
'''Tests for so_lazy custom function implementation'''

import unittest
from so_lazy import lazy

# pylint: disable=global-statement
# pylint: disable=duplicate-code

TEST_DATASET = [1, 2, 3]

_DATA = []

@lazy(loader='_custom_lazy_loader')
def _default_func_no_args():
    return len(_DATA)

@lazy(loader='_custom_lazy_loader')
def _default_func_args(inc):
    return len(_DATA) + inc

@lazy(loader='_custom_lazy_loader')
def _default_func_kwargs(**kwargs):
    inc = kwargs.get('inc2', 0)
    return len(_DATA) + inc

@lazy(loader='_custom_lazy_loader')
def _default_func_args_kwargs(inc, **kwargs):
    inc = kwargs.get('inc2', 0)
    return len(_DATA) + inc

def _lazy_loader():
    raise AttributeError('Default lazy loader was called')

def _custom_lazy_loader():
    global _DATA

    if len(_DATA) == 0:
        _DATA = TEST_DATASET

class FunctionCustomLoaderTests(unittest.TestCase):
    '''Test function lazy load with custom loader'''

    def setUp(self):
        global _DATA

        _DATA = []

    @staticmethod
    def test_data_loaded_no_args():
        '''Test custom function lazy load with no args'''

        assert len(_DATA) == 0
        assert _default_func_no_args() == len(TEST_DATASET)
        assert len(_DATA) == len(TEST_DATASET)

    @staticmethod
    def test_data_loaded_args():
        '''Test custom function lazy load with args'''

        assert len(_DATA) == 0
        assert _default_func_args(0) == len(TEST_DATASET)
        assert len(_DATA) == len(TEST_DATASET)

    @staticmethod
    def test_data_loaded_kwargs():
        '''Test custom function lazy load with kwargs'''

        assert len(_DATA) == 0
        assert _default_func_kwargs(inc2=0) == len(TEST_DATASET)
        assert len(_DATA) == len(TEST_DATASET)

    @staticmethod
    def test_data_loaded_args_kwargs():
        '''Test custom function lazy load with args and kwargs'''

        assert len(_DATA) == 0
        assert _default_func_args_kwargs(0, inc2=0) == len(TEST_DATASET)
        assert len(_DATA) == len(TEST_DATASET)




'''Tests for so_lazy class implementation'''

import unittest
from so_lazy import lazy, ClassDecoratedError, NoLoaderError

# pylint: disable=too-few-public-methods

TEST_DATASET = [1, 2, 3]

class Parent:
    '''Basic class with lazy load defaults'''

    def __init__(self):
        self.data = []

    @lazy()
    def length(self):
        '''Length of internal data'''
        return len(self.data)

    @lazy()
    def length2(self):
        '''Length 2 of internal data'''
        return len(self.data)

    def _lazy_loader(self):
        if len(self.data) == 0:
            self.data = TEST_DATASET

class Child(Parent):
    '''Child class with no extensions'''

class ChildLazyLoadFunctionDisabled(Parent):
    '''Child class with no extensions but one disabled lazy
       load function'''

    def length2(self):
        '''Override of Parent method'''
        return len(self.data)

class ChildFunctionLoader(Parent):
    '''Child class with redefined lazy function'''

    @lazy(loader='_custom_lazy_loader')
    def length2(self):
        return len(self.data)

    def _custom_lazy_loader(self):
        if len(self.data) == 0:
            self.data = TEST_DATASET + TEST_DATASET

@lazy()
class DecoratedError:
    '''Class decorated with lazy'''

class NoLazyLoader:
    '''Class without a lazy loader'''

    def __init__(self):
        self.data = []

    @lazy()
    def length(self):
        '''Length of internal data'''
        return len(self.data)

class ClassParentDefaultTests(unittest.TestCase):
    '''Test basic class lazy load with defaults'''

    def setUp(self):
        self.test_class = Parent()

    def test_data_loaded(self):
        '''Test basic class lazy load'''

        assert len(self.test_class.data) == 0
        assert self.test_class.length() == len(TEST_DATASET)
        assert len(self.test_class.data) == len(TEST_DATASET)

class ClassChildDefaultTests(unittest.TestCase):
    '''Test inherited lazy load'''

    def setUp(self):
        self.test_class = Child()

    def test_data_loaded(self):
        '''Test inherited class lazy load'''

        assert len(self.test_class.data) == 0
        assert self.test_class.length() == len(TEST_DATASET)
        assert len(self.test_class.data) == len(TEST_DATASET)

class ClassChildLazyLoadFunctionDisabledTests(unittest.TestCase):
    '''Test inherited lazy load function can be disabled'''

    def setUp(self):
        self.test_class = ChildLazyLoadFunctionDisabled()

    def test_no_data_loaded(self):
        '''Test inherited class function lazy load is disabled'''

        assert len(self.test_class.data) == 0
        assert self.test_class.length2() == 0
        assert len(self.test_class.data) == 0

    def test_data_loaded(self):
        '''Test inherited class lazy load still works for
           non-disabled functions'''

        assert len(self.test_class.data) == 0
        assert self.test_class.length() == len(TEST_DATASET)
        assert len(self.test_class.data) == len(TEST_DATASET)

class ClassChildFunctionLoaderTests(unittest.TestCase):
    '''Test inherited lazy load can be redefined per function'''

    def setUp(self):
        self.test_class = ChildFunctionLoader()

    def test_data_loaded(self):
        '''Test inherited function lazy loader not redefined'''

        assert len(self.test_class.data) == 0
        assert self.test_class.length() == len(TEST_DATASET)
        assert len(self.test_class.data) == len(TEST_DATASET)

    def test_data_custom_loaded(self):
        '''Test inherited function lazy loader redefined'''

        assert len(self.test_class.data) == 0
        assert self.test_class.length2() == len(TEST_DATASET) * 2
        assert len(self.test_class.data) == len(TEST_DATASET) * 2

class ClassErrorTests(unittest.TestCase):
    '''Test lazy errors'''

    def test_class_decorated_error(self):
        '''Test for class decorated error'''
        self.assertRaises(ClassDecoratedError, DecoratedError)

    def test_no_loader_error(self):
        '''Test for no loader error'''
        test_class = NoLazyLoader()
        self.assertRaises(NoLoaderError, test_class.length)




'''Tests for so_lazy package'''

import unittest
import os

import so_lazy

class PackageVarTests(unittest.TestCase):
    '''Test package variables are set'''

    @staticmethod
    def test_author():
        '''Package info author exists'''
        assert so_lazy.__author__ is not None

    @staticmethod
    def test_homepage():
        '''Package info homepage exists'''
        assert so_lazy.__homepage__ is not None

    @staticmethod
    def test_download():
        '''Package info download exists'''
        assert so_lazy.__download__ is not None

    @staticmethod
    def test_license():
        '''Package info license exists'''
        assert so_lazy.__license__ is not None

    @staticmethod
    def test_description():
        '''Package info description exists'''
        assert so_lazy.__description__ is not None

    @staticmethod
    def test_version():
        '''Package info version exists'''
        assert so_lazy.__version__ is not None

    @staticmethod
    def test_long_description():
        '''Package info long description exists'''
        assert os.path.isfile('./README.md')



#!/usr/bin/env python

'''Test runner'''

import os.path
import sys
import unittest

if __name__ == '__main__':
    sys.path.insert(0, os.path.realpath('..'))

    suites = unittest.defaultTestLoader.discover('.', 'test*.py')

    if not unittest.TextTestRunner(verbosity=2).run(suites).wasSuccessful():
        sys.exit(1)



#!/usr/bin/env python

'''Setup configuration for so_lazy package'''

import json
import setuptools

PKG_NAME = 'so_lazy'

with open(PKG_NAME + '/pkg_info.json') as fh:
    _pkg_info = json.load(fh)

with open('README.md', 'r') as fh:
    long_description = fh.read()

setuptools.setup(
    name=PKG_NAME,
    version=_pkg_info['version'],
    author=_pkg_info['author'],
    description=_pkg_info['description'],
    long_description=long_description,
    long_description_content_type='text/markdown',
    url=_pkg_info['homepage'],
    download_url=_pkg_info['download'],
    license=_pkg_info['license'],
    packages=setuptools.find_packages(),
    include_package_data=True,
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ],
    python_requires='>=3.5'
)




'''so_lazy lazy loading Python package'''

import json
import os

with open(os.path.dirname(__file__) + '/pkg_info.json') as handle:
    _pkg_info = json.load(handle)

__author__ = _pkg_info['author']
__homepage__ = _pkg_info['homepage']
__download__ = _pkg_info['download']
__license__ = _pkg_info['license']
__description__ = _pkg_info['description']
__version__ = _pkg_info['version']

from functools import wraps

def lazy(**dec_kwargs):
    '''Lazy loader decorator'''

    def _decorator(obj):

        @wraps(obj)
        def wrapper(*args, **kwargs):

            # check if class decorator
            if isinstance(obj, type):
                raise ClassDecoratedError('lazy is not designed to decorate classes')

            # set defaults
            loader = None
            loader_name = dec_kwargs.get('loader', '_lazy_loader')

            # check if calling class has loader
            if len(args) > 0:
                loader = getattr(args[0], loader_name, None)

            # check calling scope for loader
            if loader is None:
                loader = obj.__globals__.get(loader_name, None)

            # call lazy loader
            if loader is not None:
                loader()
            else:
                raise NoLoaderError('%s not found in current scope' % loader_name)

            return obj(*args, **kwargs)

        return wrapper

    return _decorator

# ----------
# Exceptions
# -----------

class ClassDecoratedError(TypeError):
    '''Class decorator error'''

class NoLoaderError(AttributeError):
    '''No loader found error'''
